package be.iminds.ilabt.jfed.espec.model;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * AnsibleHostSpec
 */
public class AnsibleHostSpec {
    public static enum HostType {
        /**
         * Add a node to the RSpec to run ansible on.
         */
        ADD,
        /**
         * Use an existing node in the RSpec to run ansible on.
         */
        EXISTING,
        /**
         * Use the local node, not in the experiment, to run ansible on.
         */
        LOCAL }

    @Nonnull private final HostType hostType;
    @Nullable private final String nodeName;
    @Nonnull private final List<UploadSpec> uploadLists;
    @Nonnull private final List<ExecuteSpec> executeLists;
    @Nullable private final String galaxyCommand;
    @Nullable private final String playbookCommand;

    public AnsibleHostSpec(@Nonnull HostType hostType,
                           @Nullable String nodeName,
                           @Nullable List<UploadSpec> uploadLists,
                           @Nullable List<ExecuteSpec> executeLists,
                           @Nullable String galaxyCommand,
                           @Nullable String playbookCommand) {
        this.hostType = hostType;
        this.nodeName = nodeName;
        this.uploadLists = uploadLists == null ? Collections.emptyList() : Collections.unmodifiableList(new ArrayList<>(uploadLists));
        this.executeLists = executeLists == null ? Collections.emptyList() : Collections.unmodifiableList(new ArrayList<>(executeLists));
        this.galaxyCommand = galaxyCommand;
        this.playbookCommand = playbookCommand;

        switch (hostType) {
            case ADD:
            case EXISTING:
            {
                if (nodeName == null)
                    throw new IllegalArgumentException(
                            "nodeName is required for ansible host of type "+hostType);
                break;
            }
        }
    }

    @Nonnull
    public HostType getHostType() {
        return hostType;
    }

    /**
     * The name of the ansible node (to be created or already existing)
     * nodename is never null if hostType is ADD or EXISTING
     * @return
     */
    @Nullable
    public String getNodeName() {
        return nodeName;
    }

    @Nonnull
    public List<UploadSpec> getUploadLists() {
        return uploadLists;
    }

    @Nonnull
    public List<ExecuteSpec> getExecuteLists() {
        return executeLists;
    }

    @Nullable
    public String getGalaxyCommand() {
        return galaxyCommand;
    }

    @Nullable
    public String getPlaybookCommand() {
        return playbookCommand;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (!(o instanceof AnsibleHostSpec)) return false;

        AnsibleHostSpec that = (AnsibleHostSpec) o;

        if (hostType != that.hostType) return false;
        if (nodeName != null ? !nodeName.equals(that.nodeName) : that.nodeName != null) return false;
        if (!uploadLists.equals(that.uploadLists)) return false;
        if (!executeLists.equals(that.executeLists)) return false;
        if (galaxyCommand != null ? !galaxyCommand.equals(that.galaxyCommand) : that.galaxyCommand != null)
            return false;
        return playbookCommand != null ? playbookCommand.equals(that.playbookCommand) : that.playbookCommand == null;
    }

    @Override
    public int hashCode() {
        int result = hostType.hashCode();
        result = 31 * result + (nodeName != null ? nodeName.hashCode() : 0);
        result = 31 * result + uploadLists.hashCode();
        result = 31 * result + executeLists.hashCode();
        result = 31 * result + (galaxyCommand != null ? galaxyCommand.hashCode() : 0);
        result = 31 * result + (playbookCommand != null ? playbookCommand.hashCode() : 0);
        return result;
    }

    @Override
    public String toString() {
        return "AnsibleHostSpec{" +
                "hostType=" + hostType +
                ", nodeName='" + nodeName + '\'' +
                ", uploadLists=" + uploadLists +
                ", executeLists=" + executeLists +
                ", galaxyCommand='" + galaxyCommand + '\'' +
                ", playbookCommand='" + playbookCommand + '\'' +
                '}';
    }
}
